from nox import Session, options

from nox_uv import session

options.default_venv_backend = "uv"
options.reuse_existing_virtualenvs = False

options.sessions = [
    "check_python_version",
    "install_nothing",
    "only_test_group",
    "all_groups",
    "all_extras",
    "only_one_extra",
    "correct_python",
]


@session(venv_backend="none")
def check_python_version(s: Session) -> None:
    s.run("python3", "--version")


@session
def install_nothing(s: Session) -> None:
    r = s.run("uv", "pip", "list", silent=True)
    assert isinstance(r, str)
    assert "nox-uv" in r
    assert "scapy" not in r
    assert "pyyaml" not in r
    assert "networkx" not in r
    assert "ruff" not in r
    assert "pytest-cov" not in r
    assert "mypy" not in r


@session(uv_groups=["test"])
def only_test_group(s: Session) -> None:
    r = s.run("uv", "pip", "list", silent=True)
    assert isinstance(r, str)
    assert "pytest-cov" in r
    assert "networkx" not in r
    assert "ruff" not in r


@session(uv_all_groups=True)
def all_groups(s: Session) -> None:
    r = s.run("uv", "pip", "list", silent=True)
    assert isinstance(r, str)
    assert "networkx" in r
    assert "ruff" in r
    assert "pytest-cov" in r
    assert "mypy" in r


@session(uv_all_extras=True)
def all_extras(s: Session) -> None:
    r = s.run("uv", "pip", "list", silent=True)
    assert isinstance(r, str)
    assert "networkx" not in r
    assert "scapy" in r
    assert "pyyaml" in r


@session(uv_extras=["pyyaml"])
def only_one_extra(s: Session) -> None:
    r = s.run("uv", "pip", "list", silent=True)
    assert isinstance(r, str)
    assert "networkx" not in r
    assert "pyyaml" in r


@session(python=["3.10"])
def correct_python(s: Session) -> None:
    assert s.python == "3.10"
    v = s.run("python3", "--version", silent=True)
    if isinstance(v, str):
        assert "Python 3.10" in v
    else:
        raise RuntimeError("Python version was not returned.")



import os
from pathlib import Path
import subprocess


def test_1() -> None:
    assert 5 == 5


def test_run_uv_nox() -> None:
    cur_folder = Path.cwd()
    testing_folder = Path(__file__).parent / "subproject"
    os.chdir(testing_folder)
    a = subprocess.run(["uv", "run", "python", "-m", "nox"])
    assert a.returncode == 0
    os.chdir(cur_folder)



from __future__ import annotations

from collections.abc import Callable, Sequence
import functools
from typing import Any, TypeVar, Union

import nox

R = TypeVar("R")

Python = Union[Sequence[str], str, bool]


def session(
    *args: Any,
    python: Python | None = None,
    reuse_venv: bool | None = None,
    name: str | None = None,
    venv_backend: Any | None = None,
    venv_params: Sequence[str] = (),
    tags: Sequence[str] | None = None,
    default: bool = True,
    requires: Sequence[str] | None = None,
    uv_groups: Sequence[str] = (),
    uv_extras: Sequence[str] = (),
    uv_all_extras: bool = False,
    uv_all_groups: bool = False,
    **kwargs: dict[str, Any],
) -> Callable[..., Callable[..., R]]:
    """Drop-in replacement for the :func:`nox.session` decorator to add support for `uv`.

    Args:
        args: Positional arguments are forwarded to ``nox.session``.
        kwargs: Keyword arguments are forwarded to ``nox.session``. Used to catch any future
            arguments of nox.session that aren't explicitely captured in nox_uv.session.

    Returns:
        The decorated session function.
    """
    if not args:
        return functools.partial(
            session,
            python=python,
            reuse_venv=reuse_venv,
            name=name,
            venv_backend=venv_backend,
            venv_params=venv_params,
            tags=tags,
            default=default,
            requires=requires,
            uv_groups=uv_groups,
            uv_extras=uv_extras,
            uv_all_extras=uv_all_extras,
            uv_all_groups=uv_all_groups,
            **kwargs,
        )  # type: ignore

    [function] = args

    # Create the `uv sync` command
    sync_cmd = ["uv", "sync", "--no-default-groups", "--locked"]

    # Add the groups
    for g in uv_groups:
        sync_cmd.append(f"--group={g}")

    # Add the extras
    for e in uv_extras:
        sync_cmd.append(f"--extra={e}")

    if uv_all_groups:
        sync_cmd.append("--all-groups")

    if uv_all_extras:
        sync_cmd.append("--all-extras")

    @functools.wraps(function)
    def wrapper(s: nox.Session, *_args: Any, **_kwargs: Any) -> None:
        if s.venv_backend == "uv":
            s.env["UV_PROJECT_ENVIRONMENT"] = s.virtualenv.location

            # UV called from Nox does not respect the Python version set in the Nox session.
            # We need to pass the Python version to UV explicitly.
            if s.python is not None:
                # NOTE: casting to string explicitly because implicit casting isn't working
                # for the type checker.
                s.env["UV_PYTHON"] = str(s.python)

            s.run_install(*sync_cmd)

        function(nox.Session(s._runner), *_args, **_kwargs)

    return nox.session(  # type: ignore
        wrapper,
        python=python,
        reuse_venv=reuse_venv,
        name=name,
        venv_backend=venv_backend,
        venv_params=venv_params,
        tags=tags,
        default=default,
        requires=requires,
        **kwargs,
    )



from nox import Session, options, parametrize

from nox_uv import session

options.error_on_external_run = True
options.reuse_existing_virtualenvs = False
options.default_venv_backend = "uv"
options.sessions = ["uv_lock_check", "lint", "type_check", "test"]


@session(venv_backend="none")
def uv_lock_check(s: Session) -> None:
    s.run("uv", "lock", "--check")


@session(
    python=["3.9", "3.10", "3.11", "3.12", "3.13"],
    uv_groups=["test"],
)
def test(s: Session) -> None:
    s.run(
        "python",
        "-m",
        "pytest",
        "--cov=nox_uv",
        "--cov-report=html",
        "--cov-report=term",
        "tests",
        *s.posargs,
    )


# For some sessions, set venv_backend="none" to simply execute scripts within the existing Poetry
# environment. This requires that nox is run within `poetry shell` or using `poetry run nox ...`.
@session(venv_backend="none")
@parametrize(
    "command",
    [
        # During formatting, additionally sort imports and remove unused imports.
        [
            "ruff",
            "check",
            ".",
            "--select",
            "I",
            "--select",
            "F401",
            "--extend-fixable",
            "F401",
            "--fix",
        ],
        ["ruff", "format", "."],
    ],
)
def fmt(s: Session, command: list[str]) -> None:
    s.run(*command)


@session(uv_groups=["lint"])
@parametrize(
    "command",
    [
        ["ruff", "check", "."],
        ["ruff", "format", "--check", "."],
    ],
)
def lint(s: Session, command: list[str]) -> None:
    s.run(*command)


@session(venv_backend="none")
def lint_fix(s: Session) -> None:
    s.run("ruff", "check", ".", "--extend-fixable", "F401", "--fix")


@session(venv_backend="none")
def type_check(s: Session) -> None:
    s.run("mypy", "src", "tests", "noxfile.py")


@session(venv_backend="none")
def simple_test(s: Session) -> None:
    assert 1 == 1


@session(venv_backend="none")
def run_test_as_session(s: Session) -> None:
    """Test ability to call a nother session."""
    simple_test(s)
